const error_message = "Désolé, la connexion avec Zeptobot semble être brouillée. Je vous propose de réessayer dans quelques minutes.";

/**
 * Obtient l'élément principal du chat.
 * @type {HTMLFormElement}
 */
const chatElement = document.currentScript.parentElement;
const thread = chatElement.querySelector(".thread");

/**
 * Ajoute un nouveau message dans le fil de discussion.
 * @param messageType Le type de message à insérer ("user", "bot", "error").
 * @param message Contenu du message.
 */
function appendMessage(messageType, message)
{
	// Création de l'élément du message.
	const messageElement = document.createElement("div");
	messageElement.className = messageType + " message";
	messageElement.textContent = message;

	// Insertion de l'élément du message en bas du fil de discussion.
	thread.insertBefore(messageElement, thread.querySelector("hr"));
}

/**
 * Ajoute l'animation de chargement du prochain message dans le fil de discussion.
 */
function appendLoadingElement()
{
	// Création de l'élément de chargement.
	const loadingElement = document.createElement("div");
	loadingElement.className = "writing thinking-loader";

	// Insertion de l'élément du message en bas du fil de discussion.
	thread.insertBefore(loadingElement, thread.querySelector("hr"));
}

/**
 * Supprime les éléments de chargement du prochain message.
 */
function removeLoadingElements()
{
	// On trouve tous les éléments de chargement et on les supprime tous.
	thread.querySelectorAll(".writing").forEach((element) => {
		element.remove();
	})
}

let threadId = null;

/**
 * Initialise le fil de discussion pour la page courante.
 */
function initThread()
{
	appendLoadingElement();

	// Création des données de formulaire à envoyer.
	const formData = new FormData();
	formData.set("new", "1");

	// Appelle zeptobot.
	fetch("/zeptobot", {
		method: "POST",
		body: formData,
	}).then((response) => {
		// Dans tous les cas, on supprime les éléments de chargement.
		removeLoadingElements();

		// Lecture du message texte renvoyé.
		if (response.ok)
			return response.text();
		else
			throw error_message;
	}).then((result) => {
		// Le résultat est l'ID du thread.
		threadId = result;
		// On ajoute le message d'accueil de Zeptobot après le chargement du thread.
		appendMessage(
			"bot",
			"Bonjour ! Je suis Zeptobot, votre assistant virtuel pour découvrir les solutions logicielles proposées par Zeptotech. Je suis ici pour vous aider à comprendre comment nous pouvons répondre à vos besoins en matière de développement de logiciels. Que puis-je faire pour vous ?"
		);
	}).catch((error) => {
		// Affichage de l'erreur dans la console.
		console.error(error);

		if (typeof error === "string")
			// Ajoute le message d'erreur dans le fil de discussion.
			appendMessage("error", error);
	});
}

/**
 * Envoie un message au back-end Zeptobot.
 * @param messageToSend Le message à envoyer à Zeptobot.
 */
function sendMessage(messageToSend)
{
	// Création des données de formulaire à envoyer.
	const formData = new FormData();
	formData.set("message", messageToSend);
	formData.set("thread", threadId);

	// Ajoute le message dans le fil de discussion.
	appendMessage("user", messageToSend);

	// Ajoute l'animation de chargement du message.
	appendLoadingElement();

	// Appelle zeptobot.
	fetch("/zeptobot", {
		method: "POST",
		body: formData,
	}).then((response) => {
		// Dans tous les cas, on supprime les éléments de chargement.
		removeLoadingElements();

		// Lecture du message texte renvoyé.
		if (response.ok)
			return response.text();
		else
			throw error_message;
	}).then((result) => {
		// Ajoute le message de Zeptobot dans le fil de discussion.
		appendMessage("bot", result);
	}).catch((error) => {
		// Affichage de l'erreur dans la console.
		console.error(error);

		if (typeof error === "string")
			// Ajoute le message d'erreur dans le fil de discussion.
			appendMessage("error", error);
	});
}

document.addEventListener("DOMContentLoaded", () => {
	// Initialisation d'un nouveau fil de discussion.
	initThread();

	// Récupération de la zone de rédaction du message pour Zeptobot.
	const textarea = chatElement.querySelector("textarea");

	/**
	 * Envoie un message entré dans la zone de rédaction, puis réinitialise la zone de rédaction.
	 */
	const sendTextarea = () => {
		if (threadId)
		{
			sendMessage(textarea.value);
			textarea.value = "";
		}
	};

	// À l'appui sur la touche entrée, on envoie le message rédigé.
	textarea.addEventListener("keypress", (event) => {
		if (event.key === "Enter")
		{
			event.preventDefault();
			sendTextarea();
			return false;
		}
	});

	// À la validation du formulaire, on envoie le message rédigé.
	chatElement.onsubmit = (event) => {
		event.preventDefault();
		sendTextarea();
		return false;
	};
});
